/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.ws.client;

import com.funambol.framework.engine.source.SyncSourceException;
import com.funambol.framework.filter.FilterClause;
import com.funambol.ws.engine.source.WebServiceSyncSource;
import com.funambol.ws.util.WSTools;

/**
 * Manages the interaction with the business WS.
 *
 * @version $Id: BusinessDelegate.java,v 1.1 2006/08/08 08:58:12 gibi Exp $
 */
public class BusinessDelegate implements java.io.Serializable {

    // ------------------------------------------------------------ Private data

    /** The web service sync source */
    private WebServiceSyncSource syncSource = null;

    /**
     * The WSTools used to invoke the WS
     */
    private WSTools ws = null;

    // ---------------------------------------------------------- Constructor

    /**
     * Create the BusinessDelegate and lookup the WS.
     *
     * @param syncSource the WebServiceSyncSource
     *
     * @throws SyncSourceException if an error occurs
     */
    public BusinessDelegate(WebServiceSyncSource syncSource)
    throws SyncSourceException {

        this.syncSource = syncSource;

        ws = new WSTools(syncSource.getWebServiceUrl(), null, null);

    }

    // ---------------------------------------------------------- Public Methods

    /** 
     * Calls beginSync method of the WS
     *
     * @param principalId the user and device relation identifier
     * @param syncMode the synchronization mode
     * @param filterClause the items filter clause
     *
     * @return a new session id for a sync process
     */

    public String beginSync(Long         principalId , 
                            Integer      syncMode    ,
                            FilterClause filterClause)
    throws SyncSourceException {

        Object args[] = new Object[] {principalId, syncMode, filterClause};

        return (String)ws.invoke("beginSync", args);

    }
    
    /** 
     * Calls beginSync method of the WS
     *
     * @param principalId the user and device relation identifier
     * @param syncMode the synchronization mode
     *
     * @return a new session id for a sync process
     */
    public String beginSync(Long principalId, Integer syncMode) 
    throws SyncSourceException {

        Object args[] = new Object[] {principalId, syncMode};

        return (String)ws.invoke("beginSync", args);
    }

    /** 
     * Calls getItemFromId method of the WS
     *
     * @param sessionId The session id
     * @param itemKey The item key
     *
     * @return the item with the given session id and item key or null if it
     * doesn't exists
     */
    public Item getItemFromId(String sessionId, String itemKey)
    throws SyncSourceException {

        Object args[] = new Object[] {sessionId, itemKey};

        return (Item)ws.invoke("getItemFromId", args );

    }

    /** 
     * Calls getAllItemKeys method of the WS
     *
     * @param sessionId The session id
     *
     * @return all sync item keys from the source for a given session id
     */
    public String[] getAllItemKeys(String sessionId)
    throws SyncSourceException {

            Object args[] = new Object[] {sessionId};

            return (String[])ws.invoke("getAllItemKeys", args);

    }

    /** 
     * Calls getNewItemKeys method of the WS
     *
     * @param sessionId The point of sta
     * @param since The time start point
     * @param until The time end poin
     *
     * @return sync item keys created in the time frame sinceTs - untilTs
     */
    public String[] getNewItemKeys(String sessionId, String since, String until)
    throws SyncSourceException {


            Object args[] = new Object[] {sessionId, since, until};

            return (String[])ws.invoke("getNewItemKeys", args);


    }

    /** 
     * Calls beginSync method of the WS
     *
     * @param sessionId The point of sta
     * @param since The time start point
     * @param until The time end poin
     *
     * @return sync item keys updated in the time frame sinceTs - untilTs
     */
    public String[] getUpdatedItemKeys(String sessionId, String since,
                                                         String until)
    throws SyncSourceException {

            Object args[] = new Object[] {sessionId, since, until};

            return (String[])ws.invoke("getUpdatedItemKeys", args);

    }

    /** 
     * Calls getDeletedItemKeys method of the WS
     *
     * @param sessionId The point of sta
     * @param since The time start point
     * @param until The time end poin
     *
     * @return sync item keys deleted in the time frame sinceTs - untilTs
     */
    public String[] getDeletedItemKeys(String sessionId, String since,
                                                         String until)
    throws SyncSourceException {


            Object args[] = new Object[] {sessionId, since, until};

            return (String[])ws.invoke("getDeletedItemKeys", args);

    }

    /** 
     * Calls addItem method of the WS
     *
     * @param sessionId The session id
     * @param item The new item to be stored
     * 
     * @return saved item
     */
    public Item addItem(String sessionId, Item item)
    throws SyncSourceException {

            Object args[] = new Object[] {sessionId, item};

            return (Item)ws.invoke("addItem", args);

    }

    /** 
     * Calls updateItem method of the WS
     *
     * @param sessionId The session id
     * @param item The updated item to replace the older one
     *
     * @return updated item
     */
    public Item updateItem(String sessionId, Item item)
    throws SyncSourceException {

            Object args[] = new Object[] {sessionId, item};

            return (Item)ws.invoke("updateItem", args);

    }

    /** 
     * Calls removeItem method of the WS
     *
     * @param sessionId The session id
     * @param itemKey the key of the item to remove
     * @param timestamp the time of the deletion
     * @param softDeletion is a soft delete ?
     */
    public void removeItem(String sessionId, String itemKey, String timestamp, boolean softDeletion)
    throws SyncSourceException {

            Object args[] = new Object[] {sessionId, itemKey, timestamp, Boolean.valueOf(softDeletion)};

            ws.invoke("removeItem", args);

    }    

    /** 
     * Calls endSync method of the WS
     *
     * @param sessionId The session id which will be closed
     */
    public void endSync(String sessionId) throws SyncSourceException {

            Object args[] = new Object[] {sessionId};

            ws.invoke("endSync", args);

    }

    /** 
     * Calls setOperationStatus method of the WS
     *
     * @param sessionId The session id
     * @param operation the name of the operation.
     * @param statusCode the status of the operation
     * @param itemKeys the keys of the items
     */    
    public void setOperationStatus(String sessionId, String operation, int statusCode, String[] itemKeys) 
    throws SyncSourceException{
        
            Object args[] = new Object[] {sessionId, operation, new Integer(statusCode), itemKeys};

            ws.invoke("setOperationStatus", args);
    }
    
    /** 
     * Calls getItemStateFromId method of the WS
     *
     * @param sessionId The session id
     * @param itemKey the key of the SyncItem to return
     *
     * @return char the state of the item with the given key. 
     */    
    public char getItemStateFromId(String sessionId, String itemKey) throws SyncSourceException {
        
            Object args[] = new Object[] {sessionId, itemKey};

            Byte state = (Byte)ws.invoke("getItemStateFromId", args);  
            
            return (char)state.byteValue();
            
    }


    /** 
     * Calls isItemInFilterClause method of the WS
     *
     * @param sessionId The session id
     * @param itemKey the item key to check
     *
     * @return true is the item satisfies the fiter, false otherwise
     */    
    public boolean isItemInFilterClause(String sessionId, String itemKey) throws SyncSourceException {
        
            Object args[] = new Object[] {sessionId, itemKey};

            Boolean b = (Boolean)ws.invoke("isItemInFilterClause", args);  
            
            return b.booleanValue();
    }

    /**
     * Calls mergeItems method of the WS
     * @param serverKey
     * @param clientItem
     * @return the result of the call
     * @throws SyncSourceException
     */
    public boolean mergeItems(String serverKey, Item clientItem)
    throws SyncSourceException {
        Object args[] = new Object[] {serverKey, clientItem};

        Boolean b = (Boolean) ws.invoke("mergeItems", args);

        return b.booleanValue();
    }

    /**
     * Calls getItemKeysFromTwin method of the
     * @param item
     * @return the result of the call
     */
    public String[] getItemKeysFromTwin(Item item)
            throws SyncSourceException {
        Object args[] = new Object[] {item};
        return (String[]) ws.invoke("getItemKeysFromTwin", args);
    }

}
