/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.ws.engine.source;

import com.funambol.framework.engine.SyncItem;
import com.funambol.framework.engine.SyncItemKey;
import com.funambol.framework.engine.source.MergeableSyncSource;
import com.funambol.framework.engine.source.SyncContext;
import com.funambol.framework.engine.source.SyncSourceException;
import com.funambol.framework.engine.source.AbstractSyncSource;
import com.funambol.framework.filter.FilterClause;
import com.funambol.framework.security.Sync4jPrincipal;
import com.funambol.ws.client.WebServiceSyncSourceController;

import java.io.Serializable;
import java.sql.Timestamp;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * SyncSource to sync with an web service.
 * See design document for the detail.
 *
 * @version $Id: WebServiceSyncSource.java,v 1.2 2006/08/08 11:15:22 adeac Exp $
 */
public class WebServiceSyncSource
        extends AbstractSyncSource
        implements MergeableSyncSource, Serializable {

    // --------------------------------------------------------------- Constants
    protected static final String LOGGER_NAME = "funambol.wsconnector";
    protected static final String PRODUCT_NAME = "Funambol Webservices Connector";

    // ------------------------------------------------------------ Private data
    protected transient Logger log = Logger.getLogger(LOGGER_NAME);

    // -------------------------------------------------------------- Properties
    /**
     * URL to access the WS
     */
    private String webServiceUrl = null;

    public String getWebServiceUrl() {
        return webServiceUrl;
    }

    public void setWebServiceUrl(String webServiceUrl) {
        this.webServiceUrl = webServiceUrl;
    }

    /**
     * filter, to disable or enable filtering capabilites
     */
    private boolean filter = false;

    public boolean isFilter() {
        return filter;
    }

    public void setFilter(boolean filter) {
        this.filter = filter;
    }

    /**
     * WebServiceSyncSourceController to access the controller methods
     */
    private transient WebServiceSyncSourceController controller = null;

    // ---------------------------------------------------------- Public methods

    /**
     * Returns a string representation of this object.
     *
     * @return a string representation of this object.
     */
    public String toString() {
        StringBuffer sb = new StringBuffer(super.toString());

        sb.append(" - {name: ").append(getName());
        sb.append(" type: ").append(getType());
        sb.append(" uri: ").append(getSourceURI());
        sb.append(" web service url: ").append(getWebServiceUrl());
        sb.append(" filter : ").append(isFilter());
        sb.append("}");
        return sb.toString();
    }

    /* @see com.funambol.framework.engine.source */
    public void beginSync(SyncContext context) throws SyncSourceException {
        super.beginSync(context);

        Sync4jPrincipal principal = context.getPrincipal();
        int syncMode = context.getSyncMode();
        FilterClause filter = context.getFilterClause();

        controller = new WebServiceSyncSourceController(this);

        if (isFilter())
            controller.beginSync(principal, syncMode, filter);
        else
            controller.beginSync(principal, syncMode);

    }

    /* @see com.funambol.framework.engine.source */
    public SyncItemKey[] getAllSyncItemKeys()
            throws SyncSourceException {

        try {

            return controller.getAllSyncItemKeys();

        } catch (Exception e) {
            throw new SyncSourceException("Error getting all sync item keys"
                    + e.getMessage()
            );
        }

    }

    /* @see com.funambol.framework.engine.source */
    public SyncItemKey[] getNewSyncItemKeys(Timestamp since, Timestamp until)
            throws SyncSourceException {

        try {

            return controller.getNewSyncItemKeys(since, until);

        } catch (Exception e) {
            throw new SyncSourceException("Error getting new sync item keys"
                    + e.getMessage()
            );
        }

    }

    /* @see com.funambol.framework.engine.source */
    public SyncItemKey[] getUpdatedSyncItemKeys(Timestamp since, Timestamp until)
            throws SyncSourceException {

        try {


            return controller.getUpdatedSyncItemKeys(since, until);

        } catch (Exception e) {
            throw new SyncSourceException("Error getting updated sync item keys"
                    + e.getMessage()
            );
        }

    }

    /* @see com.funambol.framework.engine.source */
    public SyncItemKey[] getDeletedSyncItemKeys(Timestamp since, Timestamp until)
            throws SyncSourceException {

        try {

            return controller.getDeletedSyncItemKeys(since, until);

        } catch (Exception e) {
            throw new SyncSourceException("Error getting deleted sync item keys"
                    + e.getMessage()
            );
        }
    }

    /* @see com.funambol.framework.engine.source */
    public SyncItem addSyncItem(SyncItem syncItem)
            throws SyncSourceException {
        try {

            return controller.addSyncItem(syncItem);

        } catch (Exception e) {
            throw new SyncSourceException(
                    "Error while adding sync item ( "
                            + syncItem.getKey().getKeyAsString() + ")"
                            + " and the cause was: ( " + e.getMessage() + ")");
        }
    }

    /* @see com.funambol.framework.engine.source */
    public SyncItem updateSyncItem(SyncItem syncItem)
            throws SyncSourceException {
        try {

            return controller.updateSyncItem(syncItem);

        } catch (Exception e) {
            throw new SyncSourceException(
                    "Error while updating sync item ( "
                            + syncItem.getKey().getKeyAsString() + ")"
                            + " and the cause was: ( " + e.getMessage() + ")");
        }
    }

    /* @see com.funambol.framework.engine.source */
    public void removeSyncItem(SyncItemKey itemKey, Timestamp timestamp, boolean softDeletion)
            throws SyncSourceException {

        try {

            controller.removeSyncItem(itemKey, timestamp, softDeletion);

        } catch (Exception e) {
            throw new SyncSourceException("Error while trying to soft remove sync item "
                    + "(" + itemKey + " , "
                    + itemKey.getKeyAsString() + ")"
                    + e.getMessage()
            );
        }

    }

    /* @see com.funambol.framework.engine.source */
    public SyncItem getSyncItemFromId(SyncItemKey syncItemKey)
            throws SyncSourceException {
        try {

            return controller.getSyncItemFromId(syncItemKey);

        } catch (Exception e) {
            throw new SyncSourceException("Error getting new item from id "
                    + e.getMessage()
            );
        }
    }

    /**
     * @see com.funambol.framework.engine.source
     */
    public boolean mergeSyncItems(SyncItemKey serverKey, SyncItem clientItem)
            throws SyncSourceException {
        try {

            return controller.mergeSyncItems(serverKey, clientItem);

        } catch (Exception e) {
            throw new SyncSourceException("Error while merging SyncItem "
                    + e.getMessage()
            );
        }
    }

    /* @see com.funambol.framework.engine.source */
    public SyncItemKey[] getSyncItemKeysFromTwin(SyncItem syncItem)
            throws SyncSourceException {
        try {

            return controller.getSyncItemKeysFromTwin(syncItem);

        } catch (Exception e) {
            throw new SyncSourceException(
                    "Error while getting sync item keys from twin "
                            + e.getMessage());
        }
    }

    /* @see com.funambol.framework.engine.source */
    public void endSync() throws SyncSourceException {
        try {

            controller.endSync();
            super.endSync();


        } catch (Exception e) {
            throw new SyncSourceException("Error while trying to end synchronization process: "
                    + e.getMessage());
        }
    }

/* @see com.funambol.framework.engine.source */

    public boolean isSyncItemInFilterClause(SyncItem item)
            throws SyncSourceException {

        return controller.isSyncItemInFilterClause(item.getKey());

    }

/* @see com.funambol.framework.engine.source */

    public boolean isSyncItemInFilterClause(SyncItemKey key)
            throws SyncSourceException {

        try {

            return controller.isSyncItemInFilterClause(key);

        } catch (Exception e) {
            throw new SyncSourceException(
                    "Error while trying to know if sync item "
                            + "with key = " + key.getKeyAsString()
                            + " is on filter clause: "
                            + e.getMessage());
        }

    }

    /**
     * @see com.funambol.framework.engine.source
     */
    public void setOperationStatus(String operation, int statusCode, SyncItemKey[] keys) {

        try {
            if (log.isLoggable(Level.FINEST)) {
                StringBuffer message = new StringBuffer("Received status code '");
                message.append(statusCode).append("' for a '").append(operation).append("'").
                        append(" for this items: ");

                for (int i = 0; i < keys.length; i++) {
                    message.append("\n- ").append(keys[i].getKeyAsString());
                }
                log.finest(message.toString());
            }

            controller.setOperationStatus(operation, statusCode, keys);

        } catch (Exception ex) {

            log.severe("Error while trying to set operation status: "
                    + ex.getMessage());
        }
    }

/* @see com.funambol.framework.engine.source */

    public char getSyncItemStateFromId(SyncItemKey key) throws SyncSourceException {

        try {

            return controller.getSyncItemStateFromId(key);

        } catch (Exception e) {
            throw new SyncSourceException(
                    "Error while trying to get sync item state: "
                            + e.getMessage());
        }

    }

}
