/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.ws.util;

import java.net.URL;
import java.net.MalformedURLException;

import java.util.logging.Logger;
import java.util.logging.Level;

import javax.xml.namespace.QName;

import org.apache.axis.AxisFault;
import org.apache.axis.client.Call;
import org.apache.axis.client.Service;

import com.funambol.framework.engine.source.SyncSourceException;
import com.funambol.framework.logging.Sync4jLogger;

/**
 * This class groups some utility methods for WS invoking and handling
 *
 * @version $Id: WSTools.java,v 1.1 2006/08/08 08:58:58 gibi Exp $
 */
public class WSTools {
    // --------------------------------------------------------------- Constants

    public static final String FAULT_UNAUTHORIZED = "(401)Unauthorized";

    public static final String FAULT_DATA_STORE   = "DataStoreException";

    // ------------------------------------------------------------ Private data

    /**
     * The WS endpoint
     */
    private URL endpoint;

    /**
     * Username for authentication
     */
    private String username;

    /**
     * Password for authentication
     */
    private String password;

    private Logger log = null;
    // ------------------------------------------------------------ Constructors

    /**
     * Creates a new WSTools with the given endpoint
     *
     * @param endpoint the endpoint
     */
    public WSTools(String endpoint, String username, String password)
    throws SyncSourceException {

        log = Sync4jLogger.getLogger("server");
        log.setLevel(Level.FINEST);

        try {
            this.endpoint = new URL(endpoint);
        } catch (MalformedURLException e) {
            throw new SyncSourceException("Incorrect admin URL '" + endpoint + "'");
        }
        this.username = username;
        this.password = password;
    }

    /**
     * Invokes the given WS method with the given objects.
     *
     * @param methodName the method to invoke
     * @param args the call arguments
     *
     * @return the object returned by the invoked method
     *
     * @throws AdminExcheption in case of SyncAdmin or administration errors
     */
    public Object invoke(final String methodName, Object[] args)
        throws SyncSourceException {

        Service  service = new Service();

        log.finest("Invoking " + methodName + " on " + endpoint);

        try {
           Call call = (Call) service.createCall();

           mapTypes(call);

           call.setTargetEndpointAddress(endpoint);
           call.setOperationName(methodName);
           call.setUsername(username);
           call.setPassword(password);

           return call.invoke(args);

        } catch (AxisFault f) {

            //
            // The exceptions are handled with the following roles:
            // 1. If there is a cause, we throw a SyncAdmin exception with
            //    CONNECTION_ERROR_MESSAGE. (Usually the cause is a connection exception).
            // 2. If the faultString is FAULT_UNAUTHORIZED we throw an UnauthorizedException
            // 3. If the faultCode is Server.userException we have to check if the exception
            //    is an WSCException or a ServerException.
            //    If it is a WSCException we get his message from the faultString and
            //    we throw a new WSCException with this message.
            //    If it is a ServerException we throw an WSCException with
            //    message UNEXPECTED_SERVER_ERROR.
            // 4. Otherwise we throw an WSCException with message UNEXPECTED_ERROR.
            //
            String errorMessage = null;

            log.severe("Axis Fault: " + f);

            Throwable cause = f.getCause();
            if (cause != null) {
                log.finest("Cause: " + f.getCause());
            }

            if (cause != null) {
                throw new SyncSourceException(Constants.CONNECTION_ERROR_MESSAGE, cause);
            }

            if (org.apache.axis.Constants.FAULT_SERVER_USER.equals(f.getFaultCode().getLocalPart())) {

               String faultString  = f.getFaultString();

               if (faultString != null) {
                   if (faultString.startsWith("com.funambol.server.admin.WSCException:")) {
                       int len = "com.funambol.server.admin.WSCException: ".length();
                       errorMessage = faultString.substring(len);
                       log.finest("errorMessage: " + errorMessage);
                   } else {
                       errorMessage = Constants.UNEXPECTED_SERVER_ERROR;
                   }
               }

               throw new SyncSourceException(errorMessage);

           } else {
               if (f.getMessage() != null && !f.getMessage().equals("")) {
                   throw new SyncSourceException(Constants.UNEXPECTED_ERROR + ": " +
                                            f.getMessage(), f);
               } else {
                   throw new SyncSourceException(Constants.UNEXPECTED_ERROR, f);
               }
           }

        } catch (Exception e) {

            log.severe("Exception: " + e);

            throw new SyncSourceException(Constants.UNEXPECTED_ERROR + ": " + e.getMessage(), e);
        }
    }


    // --------------------------------------------------------- Private methods

    /**
     * Maps the java types to the serializer-deserializer to use (the AXIS
     * BeanSerializer/Deserializer).
     * The following types are mapped:
     * <ul>
     *  <li>com.funambol.framework.security.Sync4jPrincipal</li>
     *  <li>com.funambol.framework.server.Sync4jUser</li>
     *  <li>com.funambol.framework.server.Sync4jDevice</li>
     *  <li>com.funambol.framework.server.Sync4jModule</li>
     *  <li>com.funambol.framework.config.loggingConfiguration</li>
     *  <li>com.funambol.framework.config.LoggerConfiguration</li>
     * </ul>
     *
     * @param call the call on which perform the mappings
     *
     */
    private void mapTypes(Call call) {
        mapType(
            call,
            com.funambol.ws.client.Item.class,
            "Item"
        );
        mapType(
            call,
            com.funambol.framework.filter.AllClause.class,
            "AllClause"
        );          
        mapType(
            call,
            com.funambol.framework.filter.Clause.class,
            "Clause"
        );            
        mapType(
            call,
            com.funambol.framework.filter.FieldClause.class,
            "FieldClause"
        );      
        mapType(
            call,
            com.funambol.framework.filter.FilterClause.class,
            "FilterClause"
        );          
        mapType(
            call,
            com.funambol.framework.filter.IDClause.class,
            "IDClause"
        );           
        mapType(
            call,
            com.funambol.framework.filter.LogicalClause.class,
            "LogicalClause"
        );         
        mapType(
            call,
            com.funambol.framework.filter.WhereClause.class,
            "WhereClause"
        );   
        mapType(
            call,
            com.funambol.framework.core.Property.class,
            "Property"
        );    
        mapType(
            call,
            com.funambol.framework.core.PropParam.class,
            "PropParam"
        );            
    }

    /**
     * Maps a single type.
     *
     * @param call the Call object
     * @param type the class to map
     * @param qname the type QName
     *
     */
    private void mapType(Call call, final Class type, final String qname)  {
        QName q = new QName( "urn:BeanService",  qname);
        call.registerTypeMapping(
            type,
            q,
            new org.apache.axis.encoding.ser.BeanSerializerFactory(type, q),
            new org.apache.axis.encoding.ser.BeanDeserializerFactory(type, q)
        );
    }
}
